(*******************************************************************************
 *                                                                             *
 *   Graphx : conversion vimage -> postscript                                  *
 *                                                                             *
 *******************************************************************************)

#open "graphx";;
#open "graph";;
#open "printf";;
#open "utils";;

(* sort le code postscript de [img] sur le canal [chout] *)
(* [comm] indique s'il faut placer en commentaire les commandes originales *)

let output_postscript comm chout img =

  let emit format = fprintf chout format
  and emitp format p = fprintf chout format p.x p.y
  and comment = (if comm then comment chout "%" else fun _ -> ())
  in

  (* polygone *)
  let poly(fill,v) =
    emitp "%d %d moveto\n" v.(0);
    for i=1 to vect_length(v)-1 do emitp "%d %d lineto\n" v.(i) done;
    emit "%s\n" (if fill then "fill" else "stroke");
  in

  (* courbe de Bzier *)
  let bezier(fill,v) = 
    emitp "%d %d moveto\n" v.(0);
    let i = ref 0 in
    while !i+3 < vect_length v do
      emitp "%d %d" v.(!i+1);
      emitp " %d %d"  v.(!i+2);
      emitp " %d %d curveto\n" v.(!i+3);
      i := !i + 3 
    done;
    emit "%s\n" (if fill then "fill" else "stroke");
  in

  (* chaine de caractres *)
  (* il faut chapper les parenthses et les backslashs  *)
  let string(p,s) =
    emitp "%d %d moveto (" p;
    for i=0 to string_length(s)-1 do match s.[i] with
    | (`\\` | `(` | `)` as c) -> emit "\\%c" c
    | c -> emit "%c" c
    done;
    emit ") show\n"
  in

  (* fonction gnrale de conversion *)
  let postscript(cmd) =
    comment cmd;
    match cmd with

    | Set_color(c) -> 
      let r = float_of_int(c lsr 16)/.255.0
      and g = float_of_int((c lsr 8) land 255)/.255.0
      and b = float_of_int(c land 255)/.255.0 in
      emit "%f %f %f setrgbcolor\n" r g b
 
    | Set_line_width(w) -> emit "%d setlinewidth\n" w
    | Set_font(f) -> ()
    | Set_text_size(sz) -> ()

    | Poly(x) -> poly(x)
    | Bezier(fill,d,v) ->
      if d >= 4 then bezier(fill,v) else poly(fill,poly_of_bezier d v)
    | Arc(fill,p,rx,ry,a1,a2) -> bezier(fill,bezier_of_arc(fill,p,rx,ry,a1,a2))
    | Ellipse(fill,p,rx,ry)   -> bezier(fill,bezier_of_arc(false,p,rx,ry,0,360))
    | String(x) -> string(x)
  in
 
  (* prologue *)
  emit "%%!PS-Adobe-2.0\n";
  begin
    match bounding_box img with
    | VIDE -> ()
    | BB r -> emit "%%%%BoundingBox: %d %d %d %d\n" r.xmin r.ymin r.xmax r.ymax
  end;
  (* Le bazar suivant dclare la police Helvetica en 10 points avec le *)
  (* codage iso-latin-1 ...                                            *)
  emit "/Helvetica findfont dup length dict /Helv_latin exch def\n";
  emit "{1 index /FID ne {Helv_latin 3 1 roll put} {pop pop} ifelse} forall\n";
  emit "Helv_latin /Encoding ISOLatin1Encoding put\n";
  emit "/fonte Helv_latin definefont pop\n";
  emit "/fonte findfont 10 scalefont setfont\n\n";

  (* mmet toutes les commandes *)
  do_list postscript img;

  (* pilogue *)
  emit "\nshowpage\n%%end\n"
;;



